@@ -4,7 +4,7 @@ from json import dumps
 from pathlib import Path
 from tempfile import NamedTemporaryFile
 from typing import Any, Dict, Iterator, Union, Optional
-
+from openghg.plotting import plot_timeseries as general_plot_timeseries
 import plotly.graph_objects as go
 
 from ._basedata import _BaseData
@@ -84,46 +84,15 @@ class ObsData(_BaseData, abc.Mapping):
         xlabel: Optional[str] = None,
         ylabel: Optional[str] = None,
         units: Optional[str] = None,
+        logo: Optional[bool] = True,
     ) -> go.Figure:
         """Plot a timeseries"""
 
-        species = self.metadata["species"]
-        site = self.metadata["site"]
-        inlet = self.metadata["inlet"]
-
-        if title is None:
-            title = f"{species.upper()} at {site.upper()} - {inlet}"
-
-        if xlabel is None:
-            xlabel = "Date"
-
-        if ylabel is None:
-            ylabel = "Concentration"
-
-        if units is not None:
-            ylabel += f"  ({units})"
-
-        data = self.data
-        x_data = data.time
-
-        try:
-            y_data = data[species]
-        except KeyError:
-            y_data = data["mf"]
-
-        font = {"size": 14}
-
-        title_layout = {"text": title, "y": 0.9, "x": 0.5, "xanchor": "center", "yanchor": "top"}
-
-        layout = go.Layout(
-            title=title_layout,
-            xaxis=dict(title=xlabel),
-            yaxis=dict(title=ylabel),
-            font=font,
+        return general_plot_timeseries(
+            data=self,
+            title=title,
+            xlabel=xlabel,
+            ylabel=ylabel,
+            units=units,
+            logo=logo,
         )
-
-        # Create traces
-        fig = go.Figure(layout=layout)
-        fig.add_trace(go.Scatter(x=x_data, y=y_data, mode="lines", name=species.upper()))
-
-        return fig
