@@ -2,7 +2,6 @@ import glob
 import json
 import os
 import threading
-import warnings
 from pathlib import Path
 import shutil
 from typing import Dict, List, Optional, Union
@@ -10,12 +9,13 @@ from uuid import uuid4
 import logging
 import pyvis
 from openghg.types import ObjectStoreError
+from openghg.util import read_local_config
 
 rlock = threading.RLock()
 
 __all__ = [
     "delete_object",
-    "get_local_objectstore_path",
+    "get_user_objectstore_path",
     "get_tutorial_store_path",
     "get_all_object_names",
     "get_object_names",
@@ -32,28 +32,96 @@ logger = logging.getLogger("openghg.objectstore")
 logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handler
 
 
+def get_readable_buckets() -> Dict[str, str]:
+    """Get a dictionary of readable buckets - {store_name: store_path, ...}
+
+    Returns:
+        dict: List of readable buckets
+    """
+    config = read_local_config()
+    object_stores = config["object_store"]
+
+    return {
+        store_name: data["path"] for store_name, data in object_stores.items() if "r" in data["permissions"]
+    }
+
+
+def get_writable_buckets() -> Dict[str, str]:
+    """Get a dictionary of writable buckets - {store_name: store_path, ...}
+
+    Returns:
+        dict: Dictionary of buckets this user can write to
+    """
+    config = read_local_config()
+    object_stores = config["object_store"]
+
+    return {
+        store_name: data["path"] for store_name, data in object_stores.items() if "w" in data["permissions"]
+    }
+
+
+def get_writable_bucket(name: Optional[str] = None) -> str:
+    """Get the path to a writable bucket, passing in the name of a bucket if
+    more than one writable bucket available.
+
+    Args:
+        name: Name of writable bucket
+    Returns:
+        str: Path to writable bucket
+    """
+    if os.getenv("OPENGHG_TUT_STORE") is not None:
+        return str(get_tutorial_store_path())
+
+    writable_buckets = get_writable_buckets()
+
+    if not writable_buckets:
+        raise ObjectStoreError("No writable object stores found. Check configuration file.")
+
+    if len(writable_buckets) == 1:
+        return next(iter(writable_buckets.values()))
+    elif name is not None:
+        try:
+            bucket_path = writable_buckets[name]
+        except KeyError:
+            raise ObjectStoreError(
+                f"Invalid object store name, stores you can write to are: {', '.join(writable_buckets)}"
+            )
+        return bucket_path
+    else:
+        raise ObjectStoreError(
+            f"More than one writable store, stores you can write to are: {', '.join(writable_buckets)}."
+        )
+
+
 def get_tutorial_store_path() -> Path:
     """Get the path to the local tutorial store
 
     Returns:
         pathlib.Path: Path of tutorial store
     """
-    return get_local_objectstore_path() / "tutorial_store"
+    return get_user_objectstore_path() / "tutorial_store"
 
 
 # @lru_cache
-def get_local_objectstore_path() -> Path:
-    """Read
+def get_user_objectstore_path() -> Path:
+    """Returns the path of the user's local object store
 
     Returns:
         pathlib.Path: Path of object store
     """
-    from openghg.util import read_local_config
-
     config = read_local_config()
-    object_store_path = Path(config["object_store"]["local_store"])
+    return Path(config["object_store"]["user"]["path"])
 
-    return object_store_path
+
+def get_objectstore_info() -> Dict:
+    """Read the local config file and return the data of each of the object stores the
+    user has access to.
+
+    Returns:
+        dict: Dictionary of object store data
+    """
+    config = read_local_config()
+    return config["object_store"]
 
 
 def get_all_object_names(bucket: str, prefix: Optional[str] = None, without_prefix: bool = False) -> List:
@@ -233,30 +301,29 @@ def exists(bucket: str, key: str) -> bool:
     return len(names) > 0
 
 
-def get_bucket() -> str:
-    """Find and return the local object store path (bucket)
+def get_bucket(name: Optional[str] = None) -> str:
+    """Find and return the local object store path. This will return
+    the path to the user's local object store if no name is given.
 
+    Args:
+        name: Object store name in config file
     Returns:
         str: Path to object store
     """
-    import os
+    config = read_local_config()
+
+    if name is not None:
+        try:
+            return config["object_store"][name]["path"]
+        except KeyError:
+            raise ObjectStoreError("Invalid object store name.")
 
     tutorial_store = os.getenv("OPENGHG_TUT_STORE")
+
     if tutorial_store is not None:
         return str(get_tutorial_store_path())
 
-    openghg_env = os.getenv("OPENGHG_PATH")
-    if openghg_env is not None:
-        warnings.warn(
-            "Use of the OPENGHG_PATH environment variable is deprecated and will be ignored in coming versions."
-            + "If you want to set a specific object"
-            + " store path please use the configuration file. See docs.openghg.org/install",
-            category=DeprecationWarning,
-        )
-
-        return openghg_env
-
-    local_store = get_local_objectstore_path()
+    local_store = get_user_objectstore_path()
 
     return str(local_store)
 
@@ -268,7 +335,7 @@ def clear_object_store() -> None:
     Returns:
         None
     """
-    local_store = str(get_local_objectstore_path())
+    local_store = str(get_user_objectstore_path())
     logger.warning(f"You have requested to delete {local_store}.")
 
     confirmed_path = input("Please enter the full path of the store: ")
@@ -284,6 +351,7 @@ def query_store() -> Dict:
     Returns:
         dict: Dictionary for data to be shown in force graph
     """
+    raise NotImplementedError
     from openghg.store import ObsSurface
     from openghg.store.base import Datasource
 
