@@ -1,6 +1,8 @@
 from typing import Any, Dict, List, Optional, Union
 from openghg.dataobjects import ObsData
-from openghg.util import running_on_hub
+from openghg.objectstore import get_writable_bucket
+from openghg.util import running_on_hub, load_json
+import openghg_defs
 import logging
 
 logger = logging.getLogger("openghg.retrieve")
@@ -10,13 +12,15 @@ logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handle
 def retrieve_atmospheric(
     site: str,
     species: Optional[Union[str, List]] = None,
+    inlet: Optional[str] = None,
     sampling_height: Optional[str] = None,
     start_date: Optional[str] = None,
     end_date: Optional[str] = None,
     force_retrieval: bool = False,
     data_level: int = 2,
     dataset_source: Optional[str] = None,
-    update_metadata_mismatch: bool = False,
+    store: Optional[str] = None,
+    update_mismatch: str = "never",
 ) -> Union[ObsData, List[ObsData], None]:
     """Retrieve ICOS atmospheric measurement data. If data is found in the object store it is returned. Otherwise
     data will be retrieved from the ICOS Carbon Portal. Data retrieval from the Carbon Portal may take a short time.
@@ -26,6 +30,8 @@ def retrieve_atmospheric(
     Args:
         site: Site code
         species: Species name
+        inlet: Height of the inlet for sampling in metres.
+        sampling_height: Alias for inlet
         start_date: Start date
         end_date: End date
         force_retrieval: Force the retrieval of data from the ICOS Carbon Portal
@@ -36,22 +42,28 @@ def retrieve_atmospheric(
                         This level is the ICOS-data product and free available for users.
         See https://icos-carbon-portal.github.io/pylib/modules/#stationdatalevelnone
         dataset_source: Dataset source name, for example ICOS, InGOS, European ObsPack
-        update_metadata_mismatch: If metadata derived from ICOS Header does not match
-            to derived attributes, update metadata to match to attributes.
-            Otherwise a AttrMismatchError will be raised.
+        store: Name of object to search/store data to
+        update_mismatch: This determines how mismatches between the "metadata" derived from
+            stored data and "attributes" derived from ICOS Header are handled.
+            This includes the options:
+                - "never" - don't update mismatches and raise an AttrMismatchError
+                - "from_source" / "attributes" - update mismatches based on attributes from ICOS Header
+                - "from_definition" / "metadata" - update mismatches based on input metadata
     Returns:
         ObsData, list[ObsData] or None
     """
     return retrieve(
         site=site,
         species=species,
+        inlet=inlet,
         sampling_height=sampling_height,
         start_date=start_date,
         end_date=end_date,
         force_retrieval=force_retrieval,
         data_level=data_level,
         dataset_source=dataset_source,
-        update_metadata_mismatch=update_metadata_mismatch,
+        update_mismatch=update_mismatch,
+        store=store,
     )
 
 
@@ -69,6 +81,8 @@ def retrieve(**kwargs: Any) -> Union[ObsData, List[ObsData], None]:
         species: Species name
         start_date: Start date
         end_date: End date
+        inlet: Height of the inlet for sampling in metres.
+        sampling_height: Alias for inlet
         force_retrieval: Force the retrieval of data from the ICOS Carbon Portal
         data_level: ICOS data level (1, 2)
         - Data level 1: Near Real Time Data (NRT) or Internal Work data (IW).
@@ -76,16 +90,18 @@ def retrieve(**kwargs: Any) -> Union[ObsData, List[ObsData], None]:
                         to be distributed through the Carbon Portal.
                         This level is the ICOS-data product and free available for users.
         See https://icos-carbon-portal.github.io/pylib/modules/#stationdatalevelnone
-        update_metadata_mismatch: If metadata derived from ICOS Header does not match
-            to derived attributes, update metadata to match to attributes.
-            Otherwise a AttrMismatchError will be raised.
+        update_mismatch: This determines how mismatches between the "metadata" derived from
+            stored data and "attributes" derived from ICOS Header are handled.
+            This includes the options:
+                - "never" - don't update mismatches and raise an AttrMismatchError
+                - "from_source" / "attributes" - update mismatches based on attributes from ICOS Header
+                - "from_definition" / "metadata" - update mismatches based on input metadata
     Returns:
         ObsData, list[ObsData] or None
     """
     from io import BytesIO
-
-    from openghg.cloud import call_function, unpackage
     from xarray import load_dataset
+    from openghg.cloud import call_function, unpackage
 
     # The hub is the only place we want to make remote calls
     if running_on_hub():
@@ -123,13 +139,15 @@ def retrieve(**kwargs: Any) -> Union[ObsData, List[ObsData], None]:
 def local_retrieve(
     site: str,
     species: Optional[Union[str, List]] = None,
+    inlet: Optional[str] = None,
     sampling_height: Optional[str] = None,
     start_date: Optional[str] = None,
     end_date: Optional[str] = None,
     force_retrieval: bool = False,
     data_level: int = 2,
     dataset_source: Optional[str] = None,
-    update_metadata_mismatch: bool = False,
+    store: Optional[str] = None,
+    update_mismatch: str = "never",
     **kwargs: Any,
 ) -> Union[ObsData, List[ObsData], None]:
     """Retrieve ICOS atmospheric measurement data. If data is found in the object store it is returned. Otherwise
@@ -140,6 +158,8 @@ def local_retrieve(
     Args:
         site: Site code
         species: Species name
+        inlet: Height of the inlet for sampling in metres.
+        sampling_height: Alias for inlet
         start_date: Start date
         end_date: End date
         force_retrieval: Force the retrieval of data from the ICOS Carbon Portal
@@ -150,9 +170,13 @@ def local_retrieve(
                         This level is the ICOS-data product and free available for users.
         See https://icos-carbon-portal.github.io/pylib/modules/#stationdatalevelnone
         dataset_source: Dataset source name, for example ICOS, InGOS, European ObsPack
-        update_metadata_mismatch: If metadata derived from ICOS Header does not match
-            to derived attributes, update metadata to match to attributes.
-            Otherwise a AttrMismatchError will be raised.
+        store: Name of object to search/store data to
+        update_mismatch: This determines how mismatches between the "metadata" derived from
+            stored data and "attributes" derived from ICOS Header are handled.
+            This includes the options:
+                - "never" - don't update mismatches and raise an AttrMismatchError
+                - "from_source" / "attributes" - update mismatches based on attributes from ICOS Header
+                - "from_definition" / "metadata" - update mismatches based on input metadata
     Returns:
         ObsData, list[ObsData] or None
     """
@@ -163,17 +187,23 @@ def local_retrieve(
     if not 1 <= data_level <= 2:
         logger.error("Error: data level must be 1 or 2.")
 
+    if sampling_height and inlet is None:
+        inlet = sampling_height
+    elif sampling_height and inlet:
+        logger.warning(f"Both sampling height and inlet specified. Using inlet value of {inlet}")
+
     # NOTE - we skip ranking here, will we be ranking ICOS data?
     results = search_surface(
         site=site,
         species=species,
-        sampling_height=sampling_height,
+        inlet=inlet,
         network="ICOS",
         data_source="icoscp",
         start_date=start_date,
         end_date=end_date,
         icos_data_level=data_level,
         dataset_source=dataset_source,
+        store=store,
     )
 
     if results and not force_retrieval:
@@ -185,13 +215,17 @@ def local_retrieve(
             species=species,
             data_level=data_level,
             dataset_source=dataset_source,
-            update_metadata_mismatch=update_metadata_mismatch,
+            inlet=inlet,
+            sampling_height=sampling_height,
+            update_mismatch=update_mismatch,
         )
 
         if standardised_data is None:
             return None
 
-        ObsSurface.store_data(data=standardised_data)
+        bucket = get_writable_bucket(name=store)
+        with ObsSurface(bucket=bucket) as obs:
+            obs.store_data(data=standardised_data)
 
         # Create the expected ObsData type
         obs_data = []
@@ -212,9 +246,10 @@ def _retrieve_remote(
     site: str,
     data_level: int,
     species: Optional[Union[str, List]] = None,
+    inlet: Optional[str] = None,
     sampling_height: Optional[str] = None,
     dataset_source: Optional[str] = None,
-    update_metadata_mismatch: bool = False,
+    update_mismatch: str = "never",
 ) -> Optional[Dict]:
     """Retrieve ICOS data from the ICOS Carbon Portal and standardise it into
     a format expected by OpenGHG. A dictionary of metadata and Datasets
@@ -229,11 +264,15 @@ def _retrieve_remote(
                         This level is the ICOS-data product and free available for users.
         See https://icos-carbon-portal.github.io/pylib/modules/#stationdatalevelnone
         species: Species name
-        sampling_height: Sampling height in metres
+        inlet: Height of the inlet for sampling in metres.
+        sampling_height: Alias for inlet
         dataset_source: Dataset source name, for example ICOS, InGOS, European ObsPack
-        update_metadata_mismatch: If metadata derived from ICOS Header does not match
-            to derived attributes, update metadata to match to attributes.
-            Otherwise a AttrMismatchError will be raised.
+        update_mismatch: This determines how mismatches between the "metadata" derived from
+            stored data and "attributes" derived from ICOS Header are handled.
+            This includes the options:
+                - "never" - don't update mismatches and raise an AttrMismatchError
+                - "from_source" / "attributes" - update mismatches based on attributes from ICOS Header
+                - "from_definition" / "metadata" - update mismatches based on input metadata
     Returns:
         dict or None: Dictionary of processed data and metadata if found
     """
@@ -257,6 +296,11 @@ def _retrieve_remote(
     if not isinstance(species, list):
         species = [species]
 
+    if sampling_height and inlet is None:
+        inlet = sampling_height
+    elif sampling_height and inlet:
+        logger.warning(f"Both sampling height and inlet specified. Using inlet value of {inlet}")
+
     # We should first check if it's stored in the object store
     # Will need to make sure ObsSurface can accept the datasets we
     # create from the ICOS data
@@ -272,14 +316,13 @@ def _retrieve_remote(
     species_upper = [s.upper() for s in species]
     # For this see https://stackoverflow.com/a/55335207
     search_str = r"\b(?:{})\b".format("|".join(map(re.escape, species_upper)))
-    # Now filter the dataframe so we can extraxt the PIDS
+    # Now filter the dataframe so we can extract the PIDS
     filtered_sources = data_pids[data_pids["specLabel"].str.contains(search_str)]
 
-    if sampling_height is not None:
-        sampling_height = str(float(sampling_height.rstrip("m")))
-        filtered_sources = filtered_sources[
-            [sampling_height in x for x in filtered_sources["samplingheight"]]
-        ]
+    if inlet is not None:
+        inlet = str(float(inlet.rstrip("m")))
+        height_filter = [inlet in str(x) for x in filtered_sources["samplingheight"]]
+        filtered_sources = filtered_sources[height_filter]
 
     if filtered_sources.empty:
         logger.error(
@@ -290,22 +333,27 @@ def _retrieve_remote(
     # Now extract the PIDs along with some data about them
     dobj_urls = filtered_sources["dobj"].tolist()
 
+    # Load our site metadata for a few things like the station's long_name that
+    # isn't in the ICOS metadata in the way we want it at the momenet - 2023-03-20
+    site_info_fpath = openghg_defs.site_info_file
+    openghg_site_metadata = load_json(path=site_info_fpath)
+
     standardised_data: Dict[str, Dict] = {}
 
     for n, dobj_url in enumerate(dobj_urls):
         dobj = Dobj(dobj_url)
-        # We need to pull the data down as .info (metadata) is populated further on this step
-        dataframe = dobj.get()
-        # This is the metadata, dobj.info and dobj.meta are equal
-        dobj_info = dobj.meta
-
-        metadata = {}
-
-        specific_info = dobj_info["specificInfo"]
-        col_data = specific_info["columns"]
+        logger.info(f"Retrieving {dobj_url}...")
+        # We've got to jump through some hoops here to try and avoid the NOAA
+        # ObsPack GlobalView data
+        try:
+            if "globalview" in dobj.meta["references"]["doi"]["titles"][0]["title"].lower():
+                logger.info(f"Skipping {dobj_url} as ObsPack GlobalView detected.")
+                continue
+        except KeyError:
+            pass
 
         try:
-            dobj_dataset_source = dobj_info["specification"]["project"]["self"]["label"]
+            dobj_dataset_source = dobj.meta["specification"]["project"]["self"]["label"]
         except KeyError:
             dobj_dataset_source = "NA"
             logger.warning("Unable to read project information from dobj.")
@@ -313,6 +361,16 @@ def _retrieve_remote(
         if dataset_source is not None and dataset_source.lower() != dobj_dataset_source.lower():
             continue
 
+        # We need to pull the data down as .info (metadata) is populated further on this step
+        dataframe = dobj.get()
+        # This is the metadata, dobj.info and dobj.meta are equal
+        dobj_info = dobj.meta
+
+        attributes = {}
+
+        specific_info = dobj_info["specificInfo"]
+        col_data = specific_info["columns"]
+
         # Get the species this dobj holds information for
         not_the_species = {"TIMESTAMP", "Flag", "NbPoints", "Stdev"}
         species_info = next(i for i in col_data if i["label"] not in not_the_species)
@@ -323,30 +381,30 @@ def _retrieve_remote(
 
         species_info = next(item for item in col_data if str(item["label"]).lower() == the_species.lower())
 
-        metadata["species"] = the_species
+        attributes["species"] = the_species
         acq_data = specific_info["acquisition"]
         station_data = acq_data["station"]
 
         to_store: Dict[str, Any] = {}
         try:
-            instrument_metadata = acq_data["instrument"]
+            instrument_attributes = acq_data["instrument"]
         except KeyError:
             to_store["instrument"] = "NA"
             to_store["instrument_data"] = "NA"
         else:
-            # Do some tidying of the instrument metadata
+            # Do some tidying of the instrument attributes
             instruments = set()
-            cleaned_instrument_metadata = []
+            cleaned_instrument_attributes = []
 
-            if not isinstance(instrument_metadata, list):
-                instrument_metadata = [instrument_metadata]
+            if not isinstance(instrument_attributes, list):
+                instrument_attributes = [instrument_attributes]
 
-            for inst in instrument_metadata:
+            for inst in instrument_attributes:
                 instrument_name = inst["label"]
                 instruments.add(instrument_name)
                 uri = inst["uri"]
 
-                cleaned_instrument_metadata.extend([instrument_name, uri])
+                cleaned_instrument_attributes.extend([instrument_name, uri])
 
             if len(instruments) == 1:
                 instrument = instruments.pop()
@@ -354,42 +412,68 @@ def _retrieve_remote(
                 instrument = "multiple"
 
             to_store["instrument"] = instrument
-            to_store["instrument_data"] = cleaned_instrument_metadata
+            to_store["instrument_data"] = cleaned_instrument_attributes
 
-        metadata.update(to_store)
+        attributes.update(to_store)
 
-        metadata["site"] = station_data["id"]
-        metadata["measurement_type"] = measurement_type
-        metadata["units"] = units
+        attributes["site"] = station_data["id"]
+        attributes["measurement_type"] = measurement_type
+        # TODO: Remove this from general attributes but make sure this is
+        # included as a specific value on the appropriate variable.
+        attributes["units"] = units
 
         _sampling_height = acq_data["samplingHeight"]
-        metadata["sampling_height"] = format_inlet(_sampling_height, key_name="sampling_height")
-        metadata["sampling_height_units"] = "metres"
-        metadata["inlet"] = format_inlet(_sampling_height, key_name="inlet")
-        metadata["inlet_height_magl"] = format_inlet(_sampling_height, key_name="inlet_height_magl")
+        attributes["sampling_height"] = format_inlet(_sampling_height, key_name="sampling_height")
+        attributes["sampling_height_units"] = "metres"
+        attributes["inlet"] = format_inlet(_sampling_height, key_name="inlet")
+        attributes["inlet_height_magl"] = format_inlet(_sampling_height, key_name="inlet_height_magl")
 
         loc_data = station_data["location"]
 
-        metadata["station_latitude"] = str(loc_data["lat"])
-        metadata["station_longitude"] = str(loc_data["lon"])
-        metadata["station_altitude"] = format_inlet(loc_data["alt"], key_name="station_altitude")
+        attributes["station_long_name"] = loc_data["label"]
+        attributes["station_latitude"] = str(loc_data["lat"])
+        attributes["station_longitude"] = str(loc_data["lon"])
 
-        metadata["data_owner"] = f"{stat.firstName} {stat.lastName}"
-        metadata["data_owner_email"] = str(stat.email)
-        metadata["station_height_masl"] = format_inlet(str(stat.eas), key_name="station_height_masl")
+        # 03/05/2023: Updated attributes to include altitude for "station_height_masl" explicitly.
+        # attributes["station_altitude"] = format_inlet(loc_data["alt"], key_name="station_altitude")
+        # attributes["station_height_masl"] = format_inlet(str(stat.eas), key_name="station_height_masl")
+        attributes["station_height_masl"] = format_inlet(loc_data["alt"], key_name="station_height_masl")
 
-        metadata["citation_string"] = dobj_info["references"]["citationString"]
-        metadata["licence_name"] = dobj_info["references"]["licence"]["name"]
-        metadata["licence_info"] = dobj_info["references"]["licence"]["url"]
+        attributes["data_owner"] = f"{stat.firstName} {stat.lastName}"
+        attributes["data_owner_email"] = str(stat.email)
+
+        attributes["citation_string"] = dobj_info["references"]["citationString"]
+        attributes["licence_name"] = dobj_info["references"]["licence"]["name"]
+        attributes["licence_info"] = dobj_info["references"]["licence"]["url"]
+
+        metadata = {}
+
+        network = "ICOS"
+
+        try:
+            site_info = openghg_site_metadata[site.upper()][network]
+        except KeyError:
+            pass
+        else:
+            metadata["station_long_name"] = site_info["long_name"]
+            metadata["station_latitude"] = site_info["latitude"]
+            metadata["station_longitude"] = site_info["longitude"]
+
+        # Add some values directly for attributes (for now)
+        metadata["species"] = attributes["species"]
 
         # Add ICOS in directly here for now
-        metadata["network"] = "ICOS"
-        metadata["data_type"] = "surface"
-        metadata["data_source"] = "icoscp"
-        metadata["source_format"] = "icos"
-        metadata["icos_data_level"] = str(data_level)
+        additional_data = {}
+        additional_data["network"] = network
+        additional_data["data_type"] = "surface"
+        additional_data["data_source"] = "icoscp"
+        additional_data["source_format"] = "icos"
+        additional_data["icos_data_level"] = str(data_level)
+        additional_data["dataset_source"] = dobj_dataset_source
+        additional_data["site"] = site
 
-        metadata["dataset_source"] = dobj_dataset_source
+        attributes.update(additional_data)
+        metadata.update(additional_data)
 
         dataframe.columns = [x.lower() for x in dataframe.columns]
         dataframe = dataframe.dropna(axis="index")
@@ -397,7 +481,7 @@ def _retrieve_remote(
         if not dataframe.index.is_monotonic_increasing:
             dataframe = dataframe.sort_index()
 
-        spec = metadata["species"]
+        spec = attributes["species"]
 
         rename_cols = {
             "stdev": spec + " variability",
@@ -420,7 +504,7 @@ def _retrieve_remote(
         dataframe.index = to_datetime(dataframe.index, format="%Y-%m-%d %H:%M:%S")
 
         dataset = dataframe.to_xarray()
-        dataset.attrs.update(metadata)
+        dataset.attrs.update(attributes)
 
         # So there isn't an easy way of getting a hash of a Dataset, can we do something
         # simple here we can compare data that's being added? Then we'll be able to make sure
@@ -430,12 +514,10 @@ def _retrieve_remote(
         standardised_data[data_key] = {
             "metadata": metadata,
             "data": dataset,
-            "attributes": metadata,
+            "attributes": attributes,
         }
 
-    standardised_data = assign_attributes(
-        data=standardised_data, update_metadata_mismatch=update_metadata_mismatch
-    )
+    standardised_data = assign_attributes(data=standardised_data, update_mismatch=update_mismatch)
 
     return standardised_data
 
