@@ -1,40 +1,62 @@
 from pathlib import Path
-from typing import Dict, List, Literal, Optional, Tuple, Union
+from typing import Dict, Literal, Optional, Union
+from pandas import Timedelta
 
 from openghg.cloud import create_file_package, create_post_dict
+from openghg.objectstore import get_writable_bucket
 from openghg.util import running_on_hub
-from openghg.types import optionalPathType
+from openghg.types import optionalPathType, multiPathType
 
 
 def standardise_surface(
-    filepaths: Union[str, Path, List, Tuple],
+    filepaths: multiPathType,
     source_format: str,
-    site: str,
     network: str,
+    site: str,
     inlet: Optional[str] = None,
+    height: Optional[str] = None,
     instrument: Optional[str] = None,
-    sampling_period: Optional[str] = None,
+    sampling_period: Optional[Union[Timedelta, str]] = None,
     calibration_scale: Optional[str] = None,
-    site_filepath: optionalPathType = None,
+    update_mismatch: str = "never",
+    measurement_type: str = "insitu",
     overwrite: bool = False,
+    verify_site_code: bool = True,
+    site_filepath: optionalPathType = None,
+    store: Optional[str] = None,
 ) -> Optional[Dict]:
     """Standardise surface measurements and store the data in the object store.
 
     Args:
-        filepaths: Path of file(s) to process
-        source_format: Format of data i.e. GCWERKS, CRDS, ICOS
-        site: Site code
+        filepath: Filepath(s)
+        source_format: Data format, for example CRDS, GCWERKS
+        site: Site code/name
         network: Network name
-        inlet: Inlet height in metres
+        inlet: Inlet height. Format 'NUMUNIT' e.g. "10m".
+            If retrieve multiple files pass None, OpenGHG will attempt to
+            extract this from the file.
+        height: Alias for inlet.
         instrument: Instrument name
-        sampling_period: Sampling period as pandas time code, e.g. 1m for 1 minute, 1h for 1 hour
+        sampling_period: Sampling period in pandas style (e.g. 2H for 2 hour period, 2m for 2 minute period).
+        calibration_scale: Calibration scale for data
+        update_mismatch: This determines how mismatches between the internal data
+            "attributes" and the supplied / derived "metadata" are handled.
+            This includes the options:
+                - "never" - don't update mismatches and raise an AttrMismatchError
+                - "from_source" / "attributes" - update mismatches based on input attributes
+                - "from_definition" / "metadata" - update mismatches based on input metadata
+        measurement_type: Type of measurement e.g. insitu, flask
+        overwrite: Overwrite previously uploaded data
+        verify_site_code: Verify the site code
         site_filepath: Alternative site info file (see openghg/supplementary_data repository for format).
             Otherwise will use the data stored within openghg_defs/data/site_info JSON file by default.
-        overwrite: Overwrite data currently present in the object store
+        store: Name of object store to write to, required if user has access to more than one
+        writable store
     Returns:
-        dict: Dictionary containing confirmation of standardisation process.
+        dict: Dictionary of result data
     """
     from openghg.cloud import call_function
+    from openghg.store import ObsSurface
 
     if not isinstance(filepaths, list):
         filepaths = [filepaths]
@@ -114,23 +136,120 @@ def standardise_surface(
 
         return responses
     else:
-        from openghg.store import ObsSurface
+        bucket = get_writable_bucket(name=store)
 
-        results = ObsSurface.read_file(
-            filepath=filepaths,
-            source_format=source_format,
-            site=site,
-            network=network,
-            instrument=instrument,
-            sampling_period=sampling_period,
-            inlet=inlet,
-            site_filepath=site_filepath,
-            overwrite=overwrite,
-        )
+        with ObsSurface(bucket=bucket) as obs:
+            results = obs.read_file(
+                filepath=filepaths,
+                source_format=source_format,
+                network=network,
+                site=site,
+                inlet=inlet,
+                height=height,
+                instrument=instrument,
+                sampling_period=sampling_period,
+                calibration_scale=calibration_scale,
+                measurement_type=measurement_type,
+                overwrite=overwrite,
+                verify_site_code=verify_site_code,
+                site_filepath=site_filepath,
+                update_mismatch=update_mismatch,
+            )
 
         return results
 
 
+def standardise_column(
+    filepath: Union[str, Path],
+    satellite: Optional[str] = None,
+    domain: Optional[str] = None,
+    selection: Optional[str] = None,
+    site: Optional[str] = None,
+    species: Optional[str] = None,
+    network: Optional[str] = None,
+    instrument: Optional[str] = None,
+    platform: str = "satellite",
+    source_format: str = "openghg",
+    overwrite: bool = False,
+    store: Optional[str] = None,
+) -> Optional[Dict]:
+    """Read column observation file
+
+    Args:
+        filepath: Path of observation file
+        satellite: Name of satellite (if relevant)
+        domain: For satellite only. If data has been selected on an area include the
+            identifier name for domain covered. This can map to previously defined domains
+            (see openghg_defs "domain_info.json" file) or a newly defined domain.
+        selection: For satellite only, identifier for any data selection which has been
+            performed on satellite data. This can be based on any form of filtering, binning etc.
+            but should be unique compared to other selections made e.g. "land", "glint", "upperlimit".
+            If not specified, domain will be used.
+        site : Site code/name (if relevant). Can include satellite OR site.
+        species: Species name or synonym e.g. "ch4"
+        instrument: Instrument name e.g. "TANSO-FTS"
+        network: Name of in-situ or satellite network e.g. "TCCON", "GOSAT"
+        platform: Type of platform. Should be one of:
+            - "satellite"
+            - "site"
+        source_format : Type of data being input e.g. openghg (internal format)
+        overwrite: Should this data overwrite currently stored data.
+        store: Name of store to write to
+    Returns:
+        dict: Dictionary containing confirmation of standardisation process.
+    """
+    from openghg.cloud import call_function
+    from openghg.store import ObsColumn
+
+    filepath = Path(filepath)
+
+    if running_on_hub():
+        compressed_data, file_metadata = create_file_package(filepath=filepath, obs_type="footprints")
+
+        metadata = {
+            "site": site,
+            "satellite": satellite,
+            "domain": domain,
+            "selection": selection,
+            "site": site,
+            "species": species,
+            "network": network,
+            "instrument": instrument,
+            "platform": platform,
+            "source_format": source_format,
+            "overwrite": overwrite,
+        }
+
+        metadata = {k: v for k, v in metadata.items() if v is not None}
+
+        to_post = create_post_dict(
+            function_name="standardise", data=compressed_data, metadata=metadata, file_metadata=file_metadata
+        )
+
+        fn_response = call_function(data=to_post)
+        response_content: Dict = fn_response["content"]
+        return response_content
+    else:
+        bucket = get_writable_bucket(name=store)
+
+        with ObsColumn(bucket=bucket) as obs_col:
+            result = obs_col.read_file(
+                filepath=filepath,
+                satellite=satellite,
+                domain=domain,
+                selection=selection,
+                site=site,
+                species=species,
+                network=network,
+                instrument=instrument,
+                platform=platform,
+                source_format=source_format,
+                overwrite=overwrite,
+            )
+
+        return result
+
+
 def standardise_bc(
     filepath: Union[str, Path],
     species: str,
@@ -139,6 +258,7 @@ def standardise_bc(
     period: Optional[Union[str, tuple]] = None,
     continuous: bool = True,
     overwrite: bool = False,
+    store: Optional[str] = None,
 ) -> Optional[Dict]:
     """Standardise boundary condition data and store it in the object store.
 
@@ -152,6 +272,7 @@ def standardise_bc(
         period: Period of measurements, if not passed this is inferred from the time coords
         continuous: Whether time stamps have to be continuous.
         overwrite: Should this data overwrite currently stored data.
+        store: Name of store to write to
     returns:
         dict: Dictionary containing confirmation of standardisation process.
     """
@@ -182,15 +303,19 @@ def standardise_bc(
         response_content: Dict = fn_response["content"]
         return response_content
     else:
-        return BoundaryConditions.read_file(
-            filepath=filepath,
-            species=species,
-            bc_input=bc_input,
-            domain=domain,
-            period=period,
-            continuous=continuous,
-            overwrite=overwrite,
-        )
+        bucket = get_writable_bucket(name=store)
+        with BoundaryConditions(bucket=bucket) as bcs:
+            result = bcs.read_file(
+                filepath=filepath,
+                species=species,
+                bc_input=bc_input,
+                domain=domain,
+                period=period,
+                continuous=continuous,
+                overwrite=overwrite,
+            )
+
+        return result
 
 
 def standardise_footprint(
@@ -204,12 +329,13 @@ def standardise_footprint(
     species: Optional[str] = None,
     network: Optional[str] = None,
     period: Optional[Union[str, tuple]] = None,
-    chunks: Union[int, Dict, Literal["auto"], None] = "auto",
+    chunks: Union[int, Dict, Literal["auto"], None] = None,
     continuous: bool = True,
     retrieve_met: bool = False,
     high_spatial_res: bool = False,
     high_time_res: bool = False,
     overwrite: bool = False,
+    store: Optional[str] = None,
 ) -> Optional[Dict]:
     """Reads footprint data files and returns the UUIDs of the Datasources
     the processed data has been assigned to
@@ -232,6 +358,7 @@ def standardise_footprint(
         high_time_res: Indicate footprints are high time resolution (include H_back dimension)
                         Note this will be set to True automatically for Carbon Dioxide data.
         overwrite: Overwrite any currently stored data
+        store: Name of store to write to
     Returns:
         dict / None: Dictionary containing confirmation of standardisation process. None
         if file already processed.
@@ -272,24 +399,28 @@ def standardise_footprint(
         response_content: Dict = fn_response["content"]
         return response_content
     else:
-        return Footprints.read_file(
-            filepath=filepath,
-            site=site,
-            domain=domain,
-            model=model,
-            inlet=inlet,
-            height=height,
-            metmodel=metmodel,
-            species=species,
-            network=network,
-            period=period,
-            chunks=chunks,
-            continuous=continuous,
-            retrieve_met=retrieve_met,
-            high_spatial_res=high_spatial_res,
-            high_time_res=high_time_res,
-            overwrite=overwrite,
-        )
+        bucket = get_writable_bucket(name=store)
+        with Footprints(bucket=bucket) as fps:
+            result = fps.read_file(
+                filepath=filepath,
+                site=site,
+                domain=domain,
+                model=model,
+                inlet=inlet,
+                height=height,
+                metmodel=metmodel,
+                species=species,
+                network=network,
+                period=period,
+                chunks=chunks,
+                continuous=continuous,
+                retrieve_met=retrieve_met,
+                high_spatial_res=high_spatial_res,
+                high_time_res=high_time_res,
+                overwrite=overwrite,
+            )
+
+        return result
 
 
 def standardise_flux(
@@ -305,6 +436,7 @@ def standardise_flux(
     chunks: Union[int, Dict, Literal["auto"], None] = None,
     continuous: bool = True,
     overwrite: bool = False,
+    store: Optional[str] = None,
 ) -> Optional[Dict]:
     """Process flux data
 
@@ -319,6 +451,7 @@ def standardise_flux(
         period: Period of measurements, if not passed this is inferred from the time coords
         continuous: Whether time stamps have to be continuous.
         overwrite: Should this data overwrite currently stored data.
+        store: Name of store to write to
     returns:
         dict: Dictionary of Datasource UUIDs data assigned to
     """
@@ -356,20 +489,22 @@ def standardise_flux(
         response_content: Dict = fn_response["content"]
         return response_content
     else:
-        return Emissions.read_file(
-            filepath=filepath,
-            species=species,
-            source=source,
-            domain=domain,
-            database=database,
-            database_version=database_version,
-            model=model,
-            high_time_resolution=high_time_resolution,
-            period=period,
-            continuous=continuous,
-            chunks=chunks,
-            overwrite=overwrite,
-        )
+        bucket = get_writable_bucket(name=store)
+        with Emissions(bucket=bucket) as ems:
+            return ems.read_file(
+                filepath=filepath,
+                species=species,
+                source=source,
+                domain=domain,
+                database=database,
+                database_version=database_version,
+                model=model,
+                high_time_resolution=high_time_resolution,
+                period=period,
+                continuous=continuous,
+                chunks=chunks,
+                overwrite=overwrite,
+            )
 
 
 # def upload_to_par(filepath: Optional[Union[str, Path]] = None, data: Optional[bytes] = None) -> None:
