@@ -1,3 +1,4 @@
+from __future__ import annotations
 import logging
 from collections import defaultdict
 from pathlib import Path
@@ -6,6 +7,7 @@ import numpy as np
 from openghg.store import DataSchema
 from openghg.store.base import BaseStore
 from xarray import Dataset
+from types import TracebackType
 
 __all__ = ["Footprints"]
 
@@ -20,8 +22,21 @@ class Footprints(BaseStore):
     _uuid = "62db5bdf-c88d-4e56-97f4-40336d37f18c"
     _metakey = f"{_root}/uuid/{_uuid}/metastore"
 
-    @staticmethod
-    def read_data(binary_data: bytes, metadata: Dict, file_metadata: Dict) -> Optional[Dict]:
+    def __enter__(self) -> Footprints:
+        return self
+
+    def __exit__(
+        self,
+        exc_type: Optional[BaseException],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType],
+    ) -> None:
+        if exc_type is not None:
+            logger.error(msg=f"{exc_type}, {exc_tb}")
+        else:
+            self.save()
+
+    def read_data(self, binary_data: bytes, metadata: Dict, file_metadata: Dict) -> Optional[Dict]:
         """Ready a footprint from binary data
 
         Args:
@@ -181,8 +196,8 @@ class Footprints(BaseStore):
 
     #     """
 
-    @staticmethod
     def read_file(
+        self,
         filepath: Union[str, Path],
         site: str,
         domain: str,
@@ -227,14 +242,10 @@ class Footprints(BaseStore):
         Returns:
             dict: UUIDs of Datasources data has been assigned to
         """
-        # from xarray import load_dataset
         import xarray as xr
         from openghg.store import (
-            assign_data,
-            datasource_lookup,
             infer_date_range,
             update_zero_dim,
-            load_metastore,
         )
         from openghg.util import clean_string, format_inlet, hash_file, species_lifetime, timestamp_now
 
@@ -256,15 +267,10 @@ class Footprints(BaseStore):
         inlet = format_inlet(inlet)
         inlet = cast(str, inlet)
 
-        fp = Footprints.load()
-
-        # Load in the metadata store
-        metastore = load_metastore(key=fp._metakey)
-
         file_hash = hash_file(filepath=filepath)
-        if file_hash in fp._file_hashes and not overwrite:
+        if file_hash in self._file_hashes and not overwrite:
             logger.warning(
-                f"This file has been uploaded previously with the filename : {fp._file_hashes[file_hash]} - skipping."
+                f"This file has been uploaded previously with the filename : {self._file_hashes[file_hash]} - skipping."
             )
             return None
 
@@ -368,6 +374,7 @@ class Footprints(BaseStore):
 
         # This might seem longwinded now but will help when we want to read
         # more than one footprints at a time
+        # TODO - remove this once assign_attributes has been refactored
         key = "_".join((site, domain, model, inlet))
 
         footprint_data: DefaultDict[str, Dict[str, Union[Dict, Dataset]]] = defaultdict(dict)
@@ -378,25 +385,14 @@ class Footprints(BaseStore):
         # metadata store for a Datasource, they should provide as much detail as possible
         # to uniquely identify a Datasource
         required = ("site", "model", "inlet", "domain")
-        lookup_results = datasource_lookup(metastore=metastore, data=footprint_data, required_keys=required)
 
         data_type = "footprints"
-        datasource_uuids: Dict[str, Dict] = assign_data(
-            data_dict=footprint_data,
-            lookup_results=lookup_results,
-            overwrite=overwrite,
-            data_type=data_type,
+        datasource_uuids = self.assign_data(
+            data=footprint_data, overwrite=overwrite, data_type=data_type, required_keys=required
         )
 
-        fp.add_datasources(uuids=datasource_uuids, data=footprint_data, metastore=metastore)
-
         # Record the file hash in case we see this file again
-        fp._file_hashes[file_hash] = filepath.name
-
-        fp.save()
-
-        fp_data.close()
-        metastore.close()
+        self._file_hashes[file_hash] = filepath.name
 
         return datasource_uuids
 
