@@ -1,9 +1,12 @@
+from pandas import DataFrame, Timestamp, Timedelta
+from xarray import Dataset
+from collections import defaultdict
 from typing import DefaultDict, Dict, List, Optional, Tuple, Type, TypeVar, Union
 import logging
 import numpy as np
 from openghg.store.spec import define_data_types
-from pandas import DataFrame, Timestamp
-from xarray import Dataset
+from openghg.types import ObjectStoreError
+
 
 logger = logging.getLogger("openghg.store.base")
 logger.setLevel(logging.DEBUG)  # Have to set level for logger as well as handler
@@ -24,12 +27,11 @@ class Datasource:
     _data_root = "data"
 
     def __init__(self) -> None:
-        from collections import defaultdict
+        from openghg.util import timestamp_now
         from uuid import uuid4
 
-        from openghg.util import timestamp_now
-
         self._uuid: str = str(uuid4())
+
         self._creation_datetime = timestamp_now()
         self._metadata: Dict[str, str] = {}
         # Dictionary keyed by daterange of data in each Dataset
@@ -38,9 +40,9 @@ class Datasource:
         self._start_date = None
         self._end_date = None
 
+        self._bucket = ""
+
         self._stored = False
-        # This dictionary stored the keys for each version of data uploaded
-        # data_key = d._data_keys["latest"]["keys"][date_key]
         self._data_keys: dataKeyType = defaultdict(dict)
         self._data_type: str = ""
         # Hold information regarding the versions of the data
@@ -82,6 +84,7 @@ class Datasource:
         metadata: Dict,
         data: Dataset,
         data_type: str,
+        skip_keys: Optional[List] = None,
         overwrite: Optional[bool] = False,
     ) -> None:
         """Add data to this Datasource and segment the data by size.
@@ -101,7 +104,7 @@ class Datasource:
         if data_type not in expected_data_types:
             raise TypeError(f"Incorrect data type selected. Please select from one of {expected_data_types}")
 
-        self.add_metadata(metadata=metadata)
+        self.add_metadata(metadata=metadata, skip_keys=skip_keys)
 
         if "time" in data.coords:
             return self.add_timed_data(data=data, data_type=data_type)
@@ -120,30 +123,27 @@ class Datasource:
         from openghg.util import daterange_overlap
         from xarray import concat as xr_concat
 
-        # For now we'll only group timeseries data.
-        # This is to a
-        # if data_type != "timeseries"
-        # Group by year
-        # year_group = data.groupby("time.year")
         # Extract period associated with data from metadata
         # TODO: May want to add period as a potential data variable so would need to extract from there if needed
         period = self.get_period()
 
-        new_data = {
-            self.get_representative_daterange_str(year_data, period=period): year_data
-            for _, year_data in data.groupby("time.year")
-        }
+        # Ensure data is in time order
+        time_coord = "time"
+        data = data.sortby(time_coord)
 
         # Use a dictionary keyed with the daterange covered by each segment of data
-        # new_data = {}
+        # Group by year
+        new_data = {
+            self.get_representative_daterange_str(year_data, period=period): year_data
+            for _, year_data in data.groupby(f"{time_coord}.year")
+        }
 
-        # for _, data in year_group:
-        #     daterange_str = self.get_representative_daterange_str(data, period=period)
-        #     new_data[daterange_str] = data
+        # Ensure daterange strings are independent and do not overlap each other
+        # (this can occur due to representative date strings)
+        new_data = self._clip_daterange_label(new_data)
 
         if self._data:
-            # We need to remove them from the curre
-            # pop the current daterange from
+            # We need to remove them from the current daterange
             overlapping = []
             for existing_daterange in self._data:
                 for new_daterange in new_data:
@@ -163,7 +163,7 @@ class Datasource:
                     logger.info("Combining overlapping data dateranges")
                     # Concatenate datasets along time dimension
                     try:
-                        combined = xr_concat((ex, new), dim="time")
+                        combined = xr_concat((ex, new), dim=time_coord)
                     except (ValueError, KeyError):
                         # If data variables in the two datasets are not identical,
                         # xr_concat will raise an error
@@ -174,18 +174,18 @@ class Datasource:
                         # missing variables with NaN values.
                         dv_not_in_new = dv_ex - dv_new
                         for dv in dv_not_in_new:
-                            fill_values = np.zeros(len(new["time"])) * np.nan
-                            new = new.assign({dv: ("time", fill_values)})
+                            fill_values = np.zeros(len(new[time_coord])) * np.nan
+                            new = new.assign({dv: (time_coord, fill_values)})
 
                         dv_not_in_ex = dv_new - dv_ex
                         for dv in dv_not_in_ex:
-                            fill_values = np.zeros(len(ex["time"])) * np.nan
-                            ex = ex.assign({dv: ("time", fill_values)})
+                            fill_values = np.zeros(len(ex[time_coord])) * np.nan
+                            ex = ex.assign({dv: (time_coord, fill_values)})
 
                         # Should now be able to concatenate successfully
-                        combined = xr_concat((ex, new), dim="time")
+                        combined = xr_concat((ex, new), dim=time_coord)
 
-                    combined = combined.sortby("time")
+                    combined = combined.sortby(time_coord)
 
                     unique, index, count = np_unique(combined.time, return_counts=True, return_index=True)
 
@@ -201,6 +201,10 @@ class Datasource:
                     )
                     combined_datasets[combined_daterange] = combined
 
+                # Checking for overlapping date range strings in combined
+                # data and clipping the labels as necessary.
+                combined_datasets = self._clip_daterange_label(combined_datasets)
+
                 self._data.update(combined_datasets)
             else:
                 self._data.update(new_data)
@@ -221,9 +225,10 @@ class Datasource:
         Returns:
             None
         """
-        from openghg.objectstore import delete_object, get_bucket
+        from openghg.objectstore import delete_object
 
-        bucket = get_bucket()
+        if not self._bucket:
+            raise ObjectStoreError("No bucket set, has this Datasource been previously saved?")
 
         to_delete = []
         for key_data in self._data_keys.values():
@@ -231,35 +236,20 @@ class Datasource:
             to_delete.extend(keys)
 
         for key in set(to_delete):
-            delete_object(bucket=bucket, key=key)
+            delete_object(bucket=self._bucket, key=key)
 
-    def delete_data(self, keys: List) -> None:
+    def delete_data(self, bucket: str, keys: List) -> None:
         """Delete specific keys
 
         Args:
             keys: List of keys to delete
         """
-        from openghg.objectstore import delete_object, get_bucket
-
-        bucket = get_bucket()
+        from openghg.objectstore import delete_object
 
         for key in set(keys):
             delete_object(bucket=bucket, key=key)
 
-    # def delete_version(self, version: str) -> None:
-    #     """Delete a specific version of data.
-
-    #     Args:
-    #         version: Version string
-    #     Returns:
-    #         None
-    #     """
-    #     if version not in self._data_keys:
-    #         raise KeyError("Invalid version.")
-
-    #     # keys =
-
-    def add_metadata(self, metadata: Dict) -> None:
+    def add_metadata(self, metadata: Dict, skip_keys: Optional[List] = None) -> None:
         """Add all metadata in the dictionary to this Datasource
 
         Args:
@@ -269,7 +259,7 @@ class Datasource:
         """
         from openghg.util import to_lowercase
 
-        lowercased: Dict = to_lowercase(metadata)
+        lowercased: Dict = to_lowercase(metadata, skip_keys=skip_keys)
         self._metadata.update(lowercased)
 
     def get_dataframe_daterange(self, dataframe: DataFrame) -> Tuple[Timestamp, Timestamp]:
@@ -363,6 +353,66 @@ class Datasource:
 
         return daterange_str
 
+    def clip_daterange(self, end_date: Timestamp, start_date_next: Timestamp) -> Timestamp:
+        """
+        Clip any end_date greater than the next start date (start_date_next) to be
+        1 second less.
+        """
+        if end_date >= start_date_next:
+            end_date = start_date_next - Timedelta(seconds=1)
+
+        return end_date
+
+    def clip_daterange_from_str(self, daterange_str1: str, daterange_str2: str) -> str:
+        """
+        Ensure the end date of a daterange string is not greater than the start
+        date of the next daterange string. Update as needed.
+        """
+        from openghg.util import create_daterange_str
+
+        start_date_str, end_date_str = daterange_str1.split("_")
+        start_date_next_str, _ = daterange_str2.split("_")
+
+        start_date = Timestamp(start_date_str)
+        end_date = Timestamp(end_date_str)
+        start_date_next = Timestamp(start_date_next_str)
+
+        end_date = self.clip_daterange(end_date, start_date_next)
+
+        daterange_str1_clipped = create_daterange_str(start_date, end_date)
+
+        return daterange_str1_clipped
+
+    def _clip_daterange_label(self, labelled_datasets: Dict[str, Dataset]) -> Dict[str, Dataset]:
+        """
+        Check the daterange string labels for the datasets and ensure neighbouring
+        date ranges are not overlapping. The daterange string labels will be updated
+        as required.
+
+        Args:
+            labelled_datasets : Dictionary of datasets labelled by date range strings.
+                These are expected to be in time order.
+
+        Returns:
+            Dict : Same format as input with labels updated as necessary.
+        """
+
+        datestr_labels = list(labelled_datasets.keys())
+        num_data_groups = len(datestr_labels)
+
+        labelled_datasets_clipped = {}
+        for i in range(num_data_groups):
+            daterange_str_1 = datestr_labels[i]
+            if i < num_data_groups - 1:
+                daterange_str_2 = datestr_labels[i + 1]
+                daterange_str = self.clip_daterange_from_str(daterange_str_1, daterange_str_2)
+            else:
+                daterange_str = daterange_str_1
+            dataset = labelled_datasets[daterange_str_1]
+            labelled_datasets_clipped[daterange_str] = dataset
+
+        return labelled_datasets_clipped
+
     def get_period(self) -> Optional[str]:
         """
         Extract period value from metadata. This expects keywords of either "sampling_period" (observation data) or
@@ -405,24 +455,6 @@ class Datasource:
 
         return period
 
-    @staticmethod
-    def exists(datasource_id: str, bucket: Optional[str] = None) -> bool:
-        """Check if a datasource with this ID is already stored in the object store
-
-        Args:
-            datasource_id (str): ID of datasource created from data
-        Returns:
-            bool: True if Datasource exists
-        """
-        from openghg.objectstore import exists, get_bucket
-
-        if bucket is None:
-            bucket = get_bucket()
-
-        key = f"{Datasource._datasource_root}/uuid/{datasource_id}"
-
-        return exists(bucket=bucket, key=key)
-
     def to_data(self) -> Dict:
         """Return a JSON-serialisable dictionary of object
         for storage in object store
@@ -466,17 +498,6 @@ class Datasource:
 
         return load_dataset(io.BytesIO(get_object(bucket=bucket, key=key)))
 
-        # # TODO - is there a cleaner way of doing this?
-        # with tempfile.TemporaryDirectory() as tmpdir:
-        #     tmp_path = Path(tmpdir).joinpath("tmp.nc")
-
-        #     with open(tmp_path, "wb") as f:
-        #         f.write(data)
-
-        #     ds: Dataset = xr_load_dataset(tmp_path)
-
-        #     return ds
-
     @classmethod
     def from_data(cls: Type[T], bucket: str, data: Dict, shallow: bool) -> T:
         """Construct a Datasource from JSON
@@ -491,6 +512,8 @@ class Datasource:
         from openghg.util import timestamp_tzaware
 
         d = cls()
+        # TODO: May want to merge these steps within the @classmethod
+        # into __init__() so this is not added twice.
         d._uuid = data["UUID"]
         d._creation_datetime = timestamp_tzaware(data["creation_datetime"])
         d._metadata = data["metadata"]
@@ -499,6 +522,7 @@ class Datasource:
         d._data = {}
         d._data_type = data["data_type"]
         d._latest_version = data["latest_version"]
+        d._bucket = bucket
 
         if d._stored and not shallow:
             for date_key in d._data_keys["latest"]["keys"]:
@@ -511,7 +535,7 @@ class Datasource:
 
         return d
 
-    def save(self, bucket: Optional[str] = None) -> None:
+    def save(self, bucket: str) -> None:
         """Save this Datasource object as JSON to the object store
 
         Args:
@@ -521,13 +545,9 @@ class Datasource:
         """
         from copy import deepcopy
         from pathlib import Path
-
-        from openghg.objectstore import get_bucket, set_object_from_json
+        from openghg.objectstore import set_object_from_json
         from openghg.util import timestamp_now
 
-        if bucket is None:
-            bucket = get_bucket()
-
         if self._data:
             # Ensure we have the latest key
             if "latest" not in self._data_keys:
@@ -572,6 +592,7 @@ class Datasource:
             # Link latest to the newest version
             self._data_keys["latest"] = self._data_keys[version_str]
             self._latest_version = version_str
+            self.add_metadata_key(key="latest_version", value=version_str)
 
         self._stored = True
         datasource_key = f"{Datasource._datasource_root}/uuid/{self._uuid}"
@@ -589,9 +610,8 @@ class Datasource:
     @classmethod
     def load(
         cls: Type[T],
-        bucket: Optional[str] = None,
-        uuid: Optional[str] = None,
-        key: Optional[str] = None,
+        bucket: str,
+        uuid: str,
         shallow: bool = False,
     ) -> T:
         """Load a Datasource from the object store either by name or UUID
@@ -605,22 +625,12 @@ class Datasource:
         Returns:
             Datasource: Datasource object created from JSON
         """
-        from openghg.objectstore import get_bucket, get_object_from_json
-
-        if uuid is None and key is None:
-            raise ValueError("Both uuid and key cannot be None")
-
-        if bucket is None:
-            bucket = get_bucket()
-
-        if key is None:
-            key = f"{Datasource._datasource_root}/uuid/{uuid}"
+        from openghg.objectstore import get_object_from_json
 
+        key = f"{Datasource._datasource_root}/uuid/{uuid}"
         data = get_object_from_json(bucket=bucket, key=key)
 
-        datasource = cls.from_data(bucket=bucket, data=data, shallow=shallow)
-
-        return datasource
+        return cls.from_data(bucket=bucket, data=data, shallow=shallow)
 
     def data(self) -> Dict:
         """Get the data stored in this Datasource
@@ -628,14 +638,10 @@ class Datasource:
         Returns:
             dict: Dictionary of data keyed by daterange
         """
-        from openghg.objectstore import get_bucket
-
         if not self._data:
-            bucket = get_bucket()
-
             for date_key in self._data_keys["latest"]["keys"]:
                 data_key = self._data_keys["latest"]["keys"][date_key]
-                self._data[date_key] = Datasource.load_dataset(bucket=bucket, key=data_key)
+                self._data[date_key] = Datasource.load_dataset(bucket=self._bucket, key=data_key)
 
         return self._data
 
@@ -968,3 +974,18 @@ class Datasource:
             str: Latest version
         """
         return self._latest_version
+
+    def integrity_check(self) -> None:
+        """Checks to ensure all data stored by this Datasource exists in the object store.
+
+        Returns:
+            None
+        """
+        from openghg.objectstore import exists
+
+        for version, key_data in self._data_keys.items():
+            for key in key_data["keys"].values():
+                if not exists(bucket=self._bucket, key=key):
+                    raise ObjectStoreError(
+                        f"The key {key} for version {version} of this Datasource does not exist in the object store {self._bucket}"
+                    )
